/*
 * $Id: cf_tools.c,v 1.1 2005/04/03 16:44:55 jvunder Exp $
 *
 *   OpenChanfix v2.2 
 *
 *   Channel Re-op Service Module for ircd-ratbox-2.0.7.
 *   Copyright (C) 2003-2004 Thomas Carlsson and Joost Vunderink.
 *   See http://www.garion.org/ocf/ for more information.
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 1, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */



#include "cf_base.h"

/* ----------------------------------------------------------------- */

void issue_chanfix_command_chan(struct Channel *chptr, const char *pattern, ...)
{
	va_list args;
	char tmpBuf[1024];
	va_start(args, pattern);
	vsprintf(tmpBuf, pattern, args);
	va_end(args);
	ocf_log(OCF_LOG_DEBUG, "issuecmd_chan: [%s]", tmpBuf);
	
    sendto_channel_local(ALL_MEMBERS, chptr, ":%s %s", me.name, tmpBuf);
    sendto_server(NULL, chptr, NOCAPS, NOCAPS, ":%s %s", chanfix->name, tmpBuf);

}

/* ----------------------------------------------------------------- */

void chanfix_send_privmsg(char* target, const char *pattern, ...)
{
	va_list args;
	char tmpBuf1[1024];
	char tmpBuf2[1024];
	struct Client *clptr;
    struct Channel *chptr;
    
	if (target[0] == '#') {
    	chptr = find_channel(target);
		/* Message to a channel */
		va_start(args, pattern);
		vsprintf(tmpBuf1, pattern, args);
		va_end(args);
		/* Always send PRIVMSGs to a channel, not NOTICEs. */
		snprintf(tmpBuf2, sizeof(tmpBuf2), ":%s PRIVMSG %s :%s", chanfix->name, target, tmpBuf1);
		sendto_channel_local(ALL_MEMBERS, chptr, "%s", tmpBuf2);
		sendto_server(&me, chptr, NOCAPS, NOCAPS, "%s", tmpBuf2);
		
	} else {
		/* Message to a client */
		clptr = find_client(target);
		if (!clptr) {
			return;
		}

		va_start(args, pattern);
		vsprintf(tmpBuf1, pattern, args);
		va_end(args);
		if (send_notices_instead_of_privmsgs) {
			/* NOTICE syntax depends on the target client being local */
			if (clptr->localClient) {
				sendto_one(clptr, ":%s!%s@%s NOTICE %s :%s", chanfix->name,
					chanfix->username, chanfix->host,
					clptr->name, tmpBuf1);
			} else {
				sendto_one(clptr, ":%s NOTICE %s :%s", chanfix->name,
					clptr->name, tmpBuf1);
			}
		} else {
			sendto_one(clptr, ":%s PRIVMSG %s :%s", chanfix->name,
                clptr->name, tmpBuf1);
		}
	}
}

/* ----------------------------------------------------------------- */

void chanfix_perform_mode_group(struct Channel *chptr, char plusminus, char mode, 
			char** args, int num_args)
{
	int i, j, num_modes_per_command;

	num_modes_per_command = settings_get_int("max_channel_modes");
	if (num_modes_per_command < 1) {
		num_modes_per_command = MAX_CHANNEL_MODES;
	}

	i = 0;
    /* Do as many modes as we can per line */
    while ( i < num_args ) {
		j = chanfix_perform_mode_num(chptr, plusminus, mode,
                    args + i, ( num_args - i ) < num_modes_per_command ? 
		( num_args - i ) : num_modes_per_command );

		if ( !j )
		{
			/* For some reason, we could not set this mode
			 * so we'll just skip it altogether.
			 */
			i++;
		}
		else
		{
	        i += j;
		}
    }
}

/* ----------------------------------------------------------------- */

int chanfix_perform_mode_num(struct Channel *chptr, char plusminus, char mode,
                        char** args, int number)
{
    int i;
    char cmd[IRCLINELEN + 1];
	int actualnumber;
	int linesize;
	struct Client *client_p = NULL;
	struct membership *msptr;
	
	/* "MODE #channel - " = 8 chars + channel length. */
	actualnumber = 0;
	linesize = 8 + strlen( chptr->chname );

	/* Here we will estimate how many commands we can fit into
	 * a single IRCLINELEN buffer.
 	 */
	while ( actualnumber < number && 
		( linesize + 2 + strlen( args[actualnumber] ) < 
		IRCLINELEN ) )
	{
		linesize += 2 + strlen( args[actualnumber] );
		actualnumber++;
	}

	if ( !actualnumber )
	{
		/* This should never happen, but we like
		 * old Mr Murphy and his laws!
		 */
		return 0;
	}

	/* Here we will fit the modes accordingly */
        snprintf(cmd, sizeof(cmd), "MODE %s %c", chptr->chname, plusminus);
	linesize = strlen( cmd );
    for (i = 0; i < actualnumber; i++) {
        cmd[linesize++] = mode;
    }
	cmd[linesize++] = ' ';
	cmd[linesize] = 0;

    for (i = 0; i < actualnumber; i++) 
    {
	    //check if string is user
	    if ((client_p = find_person(args[i])) != NULL)
	    {
		    //is client member of channel chptr
		    if ((msptr = find_channel_membership(chptr, client_p)) != NULL)
		    {
			    //are we opping?
			    if (plusminus == '+')
			    {
				    //if not already a chanop
			    	if (!(msptr->flags & CHFL_CHANOP))
			    	{
				    	//move to chanops list
	    				dlinkMoveNode(&msptr->channode, &chptr->members[0], &chptr->members[1]);
	    				//correct flags
	    				msptr->flags |= CHFL_CHANOP;
    				}
				}
				//or are we deopping?
	    		else if (plusminus == '-')
	    		{
		    		//if client is a chanop
	    			if (msptr->flags & CHFL_CHANOP)
	    			{
		    			//move to nonops list
	    				dlinkMoveNode(&msptr->channode, &chptr->members[1], &chptr->members[0]);
	    				//remove chanop flags
	    				msptr->flags &= ~CHFL_CHANOP;
    				}
    	    	}
	    	}
    	}
    	
        strcat(cmd, args[i]);
        strcat(cmd, " ");
    }
    
    sendto_channel_local(ALL_MEMBERS, chptr, ":%s %s", me.name, cmd);
				
    sendto_server(NULL, chptr, NOCAPS, NOCAPS, ":%s %s", chanfix->name, cmd);

	return actualnumber;
}


/* ----------------------------------------------------------------- */

void chanfix_remove_channel_modes(struct Channel *chptr)
{
	/* Remove invite-only. */
	if (chptr->mode.mode & MODE_INVITEONLY) {
		issue_chanfix_command_chan(chptr, "MODE %s -i", chptr->chname);
		chptr->mode.mode &= ~MODE_INVITEONLY;
	}

	/* Remove key. */
	if (chptr->mode.key[0] != '\0') {
		issue_chanfix_command_chan(chptr, "MODE %s -k", chptr->chname);
		memset(&chptr->mode.key, 0, KEYLEN);
	}

	/* Remove max user limit. */
	if (chptr->mode.limit != 0) {
		issue_chanfix_command_chan(chptr, "MODE %s -l", chptr->chname);
		chptr->mode.limit = 0;
	}

	//remove all bans
	if(dlink_list_length(&chptr->banlist) > 0)
		chanfix_remove_ban_list(chptr, chanfix, &chptr->banlist,
				'b', NOCAPS, ALL_MEMBERS);
	//remove all ban excempts
	if(dlink_list_length(&chptr->exceptlist) > 0)
		chanfix_remove_ban_list(chptr, chanfix, &chptr->exceptlist,
				'e', CAP_EX, ONLY_CHANOPS);
	//remove all Invite exempts
	if(dlink_list_length(&chptr->invexlist) > 0)
		chanfix_remove_ban_list(chptr, chanfix, &chptr->invexlist,
				'I', CAP_IE, ONLY_CHANOPS);
					
	return;
}

/* remove_ban_list()
 *
 * inputs	- channel, source, list to remove, char of mode, caps needed
 * outputs	-
 * side effects - given list is removed, with modes issued to local clients
 * 		  and non-TS6 servers.
 */
void
chanfix_remove_ban_list(struct Channel *chptr, struct Client *source_p,
		dlink_list *list, char c, int cap, int mems)
{
	static char lmodebuf[BUFSIZE];
	static char lparabuf[BUFSIZE];
	struct Ban *banptr;
	dlink_node *ptr;
	dlink_node *next_ptr;
	char *pbuf;
	int count = 0;
	int cur_len, mlen, plen;
	static char *mbuf;
	
	pbuf = lparabuf;

	cur_len = mlen = ircsprintf(lmodebuf, ":%s MODE %s -", 
				    source_p->name, chptr->chname);
	mbuf = lmodebuf + mlen;

	DLINK_FOREACH_SAFE(ptr, next_ptr, list->head)
	{
		banptr = ptr->data;

		/* trailing space, and the mode letter itself */
		plen = strlen(banptr->banstr) + 2;

		if(count >= MAXMODEPARAMS || (cur_len + plen) > BUFSIZE - 4)
		{
			/* remove trailing space */
			*(pbuf - 1) = '\0';

			sendto_channel_local(mems, chptr, "%s %s",
					     lmodebuf, lparabuf);
			sendto_server(source_p, chptr, cap, CAP_TS6,
				      "%s %s", lmodebuf, lparabuf);

			cur_len = mlen;
			mbuf = lmodebuf + mlen;
			count = 0;
		}

		*mbuf++ = c;
		cur_len += plen;
		pbuf += ircsprintf(pbuf, "%s ", banptr->banstr);
		count++;

		free_ban(banptr);
	}

	*(pbuf - 1) = '\0';
	sendto_channel_local(mems, chptr, "%s %s", lmodebuf, lparabuf);
	sendto_server(source_p, chptr, cap, CAP_TS6,
		      "%s %s", lmodebuf, lparabuf);

	list->head = list->tail = NULL;
	list->length = 0;
}



/* ----------------------------------------------------------------- */

void strtrim(char *string)
{
	int len = strlen(string);
	int count;

	if (len == 0) {
		return;
	}

	/* Remove whitespace at the end */
	count = len - 1;
	while (isspace(string[count]) && count > 0) {
		string[count] = 0;
		count--;
	}

	/* All-whitespace string */
	if (count == 0) {
		return;
	}

	/* Remove whitespace at the beginning */
	count = 0;
	while (isspace(string[count])) {
		count++;
	}

	if (count > 0) {
		memmove(string, string + count, len - count);
		string[len - count] = 0;
	}
}

/* ----------------------------------------------------------------- */

int readline(FILE * fp, char* line, int maxlen)
{
	int count = 0;
	char c;

	while ((c = fgetc(fp)) != EOF) {
		if (c == '\n')
		{
			line[count] = 0;
			return 1;
		}

		line[count] = c;
		count++;

		if (count >= maxlen - 1) 
		{
			line[maxlen - 1] = 0;
			return 1;
		}
	}

	return 0;
}

/* ----------------------------------------------------------------- */

int writeline(FILE * fp, char* pattern, ...)
{
	va_list args;
	char tmpBuf[1024];

	va_start(args, pattern);
	vsprintf(tmpBuf, pattern, args);
	va_end(args);
	tmpBuf[1023] = 0;

	fputs(tmpBuf, fp);
	fputc('\n', fp);
	return 1;
}

/* ----------------------------------------------------------------- */
/* Walks through array userhosts, which currently has num_userhosts
 * userhosts in it, and checks whether <user>@<host> is already
 * in there. 
 * If so, nothing is changed, and num_userhosts is returned.
 * If not, <user>@<host> is added at the end, and num_userhosts + 1
 * is returned.
 * NOTE: the userhost is copied and tolower()ed.
 */
int add_userhost_to_array(char** users, char **hosts, int num_userhosts, 
						  char* user, char* host)
{
	int i;

	/* Check if the userhost is already present. 
	 * This check is case insensitive since user@hosts are case
	 * insensitive. */
	for (i = 0; i < num_userhosts; i++) {
		if (!strcasecmp(users[i], user) && !strcasecmp(hosts[i], host)) {
			/* userhost already present. Abort and return. */
			return num_userhosts;
		}
	}

	/* Userhost is not yet in the array. Add. */
	users[num_userhosts] = user;
	hosts[num_userhosts] = host;

	return num_userhosts + 1;
}

/* ----------------------------------------------------------------- */

int get_number_of_servers_linked() 
{	
	return dlink_list_length(&global_serv_list);
}

/* ----------------------------------------------------------------- */

int check_minimum_servers_linked()
{
	int num_linked = get_number_of_servers_linked();
	
	if (100 * num_linked >= 
			settings_get_int("min_servers_present") * 
			settings_get_int("num_servers")) {
		return 1;
	}

	return 0;
}

/* ----------------------------------------------------------------- */

int tokenize_string(char *string, char *parv[OCF_MAXPARA])
{
	return tokenize_string_separator(string, parv, ' ');
}

/* ----------------------------------------------------------------- */

int tokenize_string_separator(char *string, char *parv[OCF_MAXPARA], char separator)
{
	char *p;
	char *buf = string;
	int x = 0;

	parv[x] = NULL;

	while (*buf == separator) /* skip leading spaces */
		buf++;

	if (*buf == '\0') /* ignore all-separator args */
		return x;

	do
	{
		parv[x++] = buf;
		parv[x]   = NULL;

		if ((p = strchr(buf, separator)) != NULL)
		{
			*p++ = '\0';
			buf  = p;
		}
		else
		{
			return(x);
		}

		/* Treat multiple separators as one */
		while (*buf == separator)
			buf++;

		if (*buf == '\0')
			return x;
	} while (x < OCF_MAXPARA - 1);

	parv[x++] = p;
	parv[x]   = NULL;
	return x;
}

/* ----------------------------------------------------------------- */

int get_seconds_in_current_day()
{
	time_t tt;
	struct tm *t;
	tt = time(NULL);
	t = localtime(&tt);
	return t->tm_hour * 3600 + t->tm_min * 60 + t->tm_sec;
}

/* ----------------------------------------------------------------- */

int get_days_since_epoch()
{
	return time(NULL) / 86400;
}

/* ----------------------------------------------------------------- */

char* seconds_to_datestring(time_t seconds)
{
	static char datetimestring[OCF_TIMESTAMP_LEN];
	struct tm *stm;

	stm = localtime(&seconds);
	memset(datetimestring, 0, OCF_TIMESTAMP_LEN);
	strftime(datetimestring, OCF_TIMESTAMP_LEN, "%Y-%m-%d %H:%M:%S", stm);

	return datetimestring;
}

/* slightly different from the above function - it only returns date */

char* seconds_to_date(time_t seconds)
{
	static char datetimestring[OCF_TIMESTAMP_LEN];
	struct tm *stm;

	stm = localtime(&seconds);
	memset(datetimestring, 0, OCF_TIMESTAMP_LEN);
	strftime(datetimestring, OCF_TIMESTAMP_LEN, "%Y-%m-%d", stm);

	return datetimestring;
}

/* ----------------------------------------------------------------- */

char* get_logged_in_user_string(struct ocf_loggedin_user *olu)
{
#define USER_STRING_LENGTH 80
	static char user_string[USER_STRING_LENGTH];
	memset(user_string, 0, USER_STRING_LENGTH);

	snprintf(user_string, USER_STRING_LENGTH, "%s (%s@%s) [%s]", 
		olu->nick, olu->user, olu->host,
		olu->ocf_user_p->name);

	return user_string;
}

/* ----------------------------------------------------------------- */

